package com.redhat.thermostat.tools.eclipse.plugin.editor;

import java.net.MalformedURLException;
import java.net.URL;

import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.forms.IManagedForm;
import org.eclipse.ui.forms.editor.FormEditor;
import org.eclipse.ui.forms.editor.FormPage;
import org.eclipse.ui.forms.events.HyperlinkAdapter;
import org.eclipse.ui.forms.events.HyperlinkEvent;
import org.eclipse.ui.forms.widgets.FormText;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.forms.widgets.ScrolledForm;
import org.eclipse.ui.forms.widgets.Section;
import org.eclipse.ui.forms.widgets.TableWrapLayout;

public class OverviewPage extends FormPage {

    static final String ID = "com.redhat.thermostat.tools.eclipse.plugin.editor.OverviewPage";
    static final String TITLE = "Overview";

    private String pluginName;

    public OverviewPage(FormEditor editor, String pluginName) {
        super(editor, ID, TITLE);

        this.pluginName = pluginName;
    }

    @Override
    protected void createFormContent(IManagedForm managedForm) {
        ScrolledForm form = managedForm.getForm();
        form.setText("Overview: " + pluginName);
        TableWrapLayout layout = new TableWrapLayout();
        form.getBody().setLayout(layout);

        createOverviewSection(form.getBody());

        createEditCommandsAndExtensionsSection(form.getBody());
    }


    private void createOverviewSection(Composite parent) {
        FormToolkit toolkit = getManagedForm().getToolkit();
        Section section = toolkit.createSection(parent, Section.TITLE_BAR);
        section.setText("General Information");

        Composite sectionData = toolkit.createComposite(section);
        sectionData.setLayout(new TableWrapLayout());
        FormText helpText = toolkit.createFormText(sectionData, false);
        String onlineHelp = "<form>"
                + "<p>For more information about thermostat see "
                + "<a href=\"http://icedtea.classpath.org/thermostat\">the homepage</a>. "
                + "Additional information about plugins is available "
                + "<a href=\"http://icedtea.classpath.org/wiki/Thermostat/ExtensionTutorial\">here</a>."
                + "</p>"
                + "</form>";
        helpText.setText(onlineHelp, true, false);
        helpText.addHyperlinkListener(new HyperlinkAdapter() {
            @Override
            public void linkActivated(HyperlinkEvent e) {
                String href = (String) e.getHref();
                try {
                    PlatformUI.getWorkbench().getBrowserSupport().getExternalBrowser().openURL(new URL(href));
                } catch (PartInitException | MalformedURLException e1) {
                    e1.printStackTrace();
                }
            }
        });

        section.setClient(sectionData);
    }

    private void createEditCommandsAndExtensionsSection(Composite parent) {
        FormToolkit toolkit = getManagedForm().getToolkit();
        Section section = toolkit.createSection(parent, Section.TITLE_BAR);
        section.setText("Commands and Extensions");

        Composite sectionData = toolkit.createComposite(section);
        sectionData.setLayout(new TableWrapLayout());

        FormText commandAndExtensionsForm = toolkit.createFormText(sectionData, false);
        String formContents = ""
                + "<form>"
                + "<p>Plugins provide new commands or extend existing ones.</p>"
                + "<li><a href=\"commands\">Commands</a>: declares new commands provided by this plugin</li>"
                + "<li><a href=\"extensions\">Extensions</a>: declares which commands provided by other plugins or thermostat itself should be extended</li>"
                + "</form>";
        commandAndExtensionsForm.setText(formContents, true, false);

        commandAndExtensionsForm.addHyperlinkListener(new HyperlinkAdapter() {
            @Override
            public void linkActivated(HyperlinkEvent e) {
                String href = (String) e.getHref();
                String id = null;
                switch(href) {
                    case "commands":
                        id = CommandsPage.ID;
                        break;
                    case "extensions":
                        id = ExtensionsPage.ID;
                        break;
                }
                OverviewPage.this.getEditor().setActivePage(id);
            }
        });

        section.setClient(sectionData);
    }

}
